#!/usr/bin/perl -w

#*****************************************************************************
#!
#! FILE NAME  : instgcclibs.pl
#!
#! PARAMETERS : None.
#!
#! DESCRIPTION: Find and copy the libraries that are distributed with the
#!              compiler.
#!
#! FUNCTIONS  : None.
#!
#!----------------------------------------------------------------------------
#! HISTORY
#!
#! $Log: instgcclibs.pl,v $
#! Revision 1.16  2006-09-20 14:49:43  karljope
#! ld.so.1 need x flag set, no harm to set it for all .so libs
#!
#! Revision 1.15  2006/09/15 14:35:24  karljope
#! Install striped libs instead of copying them
#!
#! Revision 1.14  2003/02/03 15:40:49  starvik
#! Made '=' after libraries: optional (now works with ARM compiler)
#!
#! Revision 1.13  2002/09/26 14:59:24  hp
#! Generalize to handle multiple library directories, as in post-gcc-2.96
#!
#! Revision 1.12  2001/10/12 16:58:56  johana
#! Removed -a from cp (we want a copy, not links)
#!
#! Revision 1.11  2001/10/12 15:32:19  johana
#! Added -af flag to cp so that links are copied and files with strange
#! permissions can be overwritten.
#!
#! Revision 1.10  2001/05/04 13:08:46  hp
#! Don't try and use shopt and stuff which may not be present.
#! Just check if libc.so exists before trying to copy .so libraries.
#!
#! Revision 1.9  2001/05/04 08:38:32  pkj
#! Make sure nullglob is set as option to the shell before trying to
#! run cp (it will fail otherwise if there are no *.so* files...)
#!
#! Revision 1.8  2001/05/02 18:17:24  hp
#! Also copy shlibs.  Add blurb explaining why we take the simple route.
#!
#! Revision 1.7  2000/12/12 21:42:47  hp
#! Fail if copying libraries fails
#!
#! Revision 1.6  2000/11/10 09:23:09  johana
#! Changed regexp to handle gcc-cris correctly
#!
#! Revision 1.5  2000/11/06 15:46:15  pkj
#! Major clean-up. Does not create a temporary file any more.
#!
#!----------------------------------------------------------------------------
#! Copyright (C) 2000, Axis Communications AB, LUND, SWEDEN
#!****************************************************************************
# $Id: instgcclibs.pl,v 1.16 2006-09-20 14:49:43 karljope Exp $

#****************** INCLUDE FILES SECTION ************************************

use strict;
use File::Basename;

#****************** VARIABLE DECLARATION SECTION *****************************

use vars qw();

#****************** CONSTANT SECTION *****************************************

#****************** MAIN PROGRAM SECTION *************************************

if ($#ARGV != 0 || ! -d $ARGV[0])
{
  print "Usage: ", basename($0), " <installation directory>\n";
  exit 1;
}

my $compiler = "gcc-cris";

my $cc = $ENV{CC} || $compiler;
my $cflags = $ENV{CFLAGS} || "";
my $ldflags = $ENV{LDFLAGS} || "";

if ($cc =~ /^([\w.\/\-\_]+)/)
{
  my $tmp = $1;
  $compiler = $tmp unless ($tmp =~ /gcc_cris$/);
}

# Find the version of the compiler

my $version = `$cc --version`;
chomp $version;

# Find the release of the compiler (in case it is gcc-cris)

my $tmp = `$cc -v -x c -c /dev/null -o /dev/null 2>&1`;
my $release = ($tmp =~ /Axis [^]]* (R.+)\]/ ? " (Release: $1)" : "");

# Find the path and name of the compiler

my $path = `which $compiler`;
$path = dirname($path);
$compiler = basename($compiler);

# Find the path to the libraries accompanying the compiler

my $libpaths = `$cc $cflags $ldflags -print-search-dirs | sed -ne 's/^libraries: =\\?\\(.*\\)/\\1/p'`;
chomp $libpaths;
my @libpaths = split ':', $libpaths;
# Library directory dumps for old versions were incorrect; the "linux"
# multilib directory component is not reflected there.  Work around that by
# prepending the directory for libgcc.a as the one to use.  It holds all
# libraries for those versions.
unshift @libpaths, dirname(`$cc $cflags $ldflags -print-file-name=libgcc.a`);

# Print the resulting information

print "## Compiler: $compiler, Version: $version$release, Path: $path\n";
print "## Libpath: $libpaths\n";

# Copy all required libraries.  We copy archives (.a) as well as shlibs
# (.so*) *resolving symlinks* (copying the actual file, not the link).

# Unless -static is given, what happens at link-time is that we link an
# app with e.g. libc.so.  The exeutable will have a tag identifying the
# needed library (the SONAME) which looks like "libc.so.6" and is the name
# of the file linked to at runtime.  It is sometimes a symlink pointing to
# the same file as (the symlink for) libc.so (for example), sometimes the
# equivalent of libc.so points to libc.so.6 pointing further to the real
# file.  In the end, there must be a file named libc.so.6 in /lib.  By
# resolving symlinks we will have libc.so.6 readily as a file, rather than
# (sometimes) a symlink to some other file.
#
# An alternative (which would save a few megs disk) would be to copy
# libraries while preserving symlinks, and later on either copy just
# libc.so.6 as a file, or play further symlink games in the target.  The
# exact SONAME for a library can in the general case is hardly guessable,
# and must be found out by inspecting the library with a tool such as
# readelf.  That'd just not be as simple as what we do here, copying
# through the symlinks at the expense of a few megs disk.
#
# We rely on that the SONAME for any library ends in .so*, which holds for
# at least all known system libraries.
#
# Things are a bit further complicated by having to cope with multiple
# search dirs, where there may be relative paths to the same directory.  We
# will do a few unnecessary copies here.  The order we copy libs in is
# reversed wrt. the search order, effectively keeping the precedence (though
# multiple same-named libraries are unlikely).  Note the hack above, though.

foreach my $libpath (reverse @libpaths)
{
  # Only try to copy the kind of libs that exist in this directory.

    my @files = glob "$libpath/*.a";
    foreach my $libfile (@files) {
	system("$ENV{INSTALL} -m 644 $libfile $ARGV[0]") == 0
	    || die ("Could not install $libfile to $ARGV[0]\n");
    }

    my @files = glob "$libpath/*.so*";
    foreach my $libfile (@files) {
	system("$ENV{INSTALL} -m 755 $libfile $ARGV[0]") == 0
	    || die ("Could not install $libfile to $ARGV[0]\n");
    }

}

#****************** FUNCTION DEFINITION SECTION ******************************

#****************** END OF FILE instgcclibs.pl *******************************
